/* $Id$

This file is part of libmspnet
Copyright © 2008  Mikkosoft Productions, Mikko Rasa
Distributed under the LGPL
*/

#include <cerrno>
#include <msp/core/refptr.h>
#include <msp/strings/formatter.h>
#include "streamlistensocket.h"
#include "streamsocket.h"

namespace Msp {
namespace Net {

StreamListenSocket::StreamListenSocket(Family af, int proto):
	Socket(af, SOCK_STREAM, proto),
	listening(false)
{ }

int StreamListenSocket::connect(const SockAddr &)
{
	throw Exception("Can't connect a listen socket");
}

void StreamListenSocket::listen(const SockAddr &addr, unsigned backlog)
{
	bind(addr);

	int err=::listen(handle, backlog);
	if(err==-1)
		throw SystemError("Unable to listen", errno);

#ifdef WIN32
	WSAEventSelect(handle, event, FD_ACCEPT);
#endif
	set_events(IO::P_INPUT);

	listening=true;
}

StreamSocket *StreamListenSocket::accept()
{
	if(!listening)
		throw InvalidState("Socket is not listening");

	sockaddr sa;
	socklen_t size=sizeof(sockaddr);
	SocketHandle new_h=::accept(handle, &sa, &size);

	RefPtr<SockAddr> paddr=SockAddr::create(sa);
	return new StreamSocket(new_h, *paddr);
}

} // namespace Net
} // namespace Msp
